﻿
using System.Windows;
using System.Windows.Media;
using MindFusion.DataViews.Wpf;
using System.ComponentModel;
using System.Windows.Controls;
using System.Windows.Data;
using System;
using System.Globalization;
using System.Windows.Media.Imaging;


namespace WpfApplication7
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			Data[] data = new Data[]
			{
				new Data()
				{
					Auth = Colors.Red,
					Site = "C017",
					Patient = "Bellic, Nikko",
					Drug = "Zosyn 4.5g q6h x6 days thru 11/24",
					Qty = 24,
					Type = DataType.ChangeRx,
					Status = DataStatus.Confirmed,
				},
				new Data()
				{
					Auth = Colors.Green,
					Site = "C017",
					Patient = "Clinton, Franklin",
					Drug = "Ertapenem 500mg x2 doses weekly",
					Qty = 2,
					Type = DataType.Refill,
					Status = DataStatus.Confirmed,
					HasAttachment = true,
					HasPrescription = true,
				},
				new Data()
				{
					Auth = Colors.Orange,
					Site = "C017",
					Patient = "Phillips, Trevor",
					Drug = "Ceftriaxone 2mg q24h x4 days",
					Qty = 4,
					Type = DataType.Refill,
					Status = DataStatus.Confirmed,
					HasAttachment = true,
					HasPrescription = true,
				},
				new Data()
				{
					Auth = Colors.Green,
					Site = "C017",
					Patient = "Johnson, Carl",
					Drug = "Daptomycin 350mg q24h x7 days",
					Qty = 7,
					Type = DataType.New,
					Status = DataStatus.Cancelled,
				},
				new Data()
				{
					Auth = Colors.Green,
					Site = "C018",
					Patient = "Speed, Claude",
					Drug = "Daptomycin 750mg q24h x7 days",
					Qty = 7,
					Type = DataType.Refill,
					Status = DataStatus.Confirmed,
					HasAttachment = true,
				},
				new Data()
				{
					Auth = Colors.Green,
					Site = "C018",
					Patient = "Lawson, Miranda",
					Drug = "Vancomycin 1800mg q12h x7 days",
					Qty = 14,
					Type = DataType.Extend,
					Status = DataStatus.Rescheduled,
				},
				new Data()
				{
					Auth = Colors.Green,
					Site = "C018",
					Patient = "Taylor, Jacob",
					Drug = "Invanz 1g q24h x4 days",
					Qty = 4,
					Type = DataType.ChangeSch,
					Status = DataStatus.Confirmed,
					HasAttachment = true,
					HasPrescription = true,
				},
			};

			grid.ColumnCreating += (s, e) =>
			    {
			        switch (e.Property.Name)
			        {
			            case "Auth":
			                e.Column = new AuthColumn(e.Property);
			                break;
			            case "Site":
			                e.Column = new SiteColumn(e.Property);
			                break;
			            case "Patient":
			                e.Column = new PatientColumn(e.Property);
			                break;
			            case "Drug":
			                e.Column = new DrugColumn(e.Property);
			                break;
						case "Qty":
							e.Column = new QtyColumn(e.Property);
							break;
						case "Type":
							e.Column = new TypeColumn(e.Property);
							break;
						case "Status":
							e.Column = new StatusColumn(e.Property);
							break;
						default:
							e.Column = null;
							break;
			        }
			    };

			grid.ItemsSource = data;

			grid.Columns[0].Width = 40;
			grid.Columns[1].Width = 60;
			grid.Columns[2].Width = 140;
			grid.Columns[3].Width = 300;
			grid.Columns[4].Width = 50;
			grid.Columns[5].Width = 90;
			grid.Columns[6].Width = 100;
		}
	}

	static class GridColumnExtensions
	{
		public static object GetPropertyValue(this GridColumn column, object rowData)
		{
			var descriptor = column.PropertyInfo.Descriptor as PropertyDescriptor;
			if (descriptor == null)
				return null;

			return descriptor.GetValue(rowData);
		}
	}

	class AuthColumn : GridColumn
	{
		public AuthColumn(ItemPropertyInfo info)
			: base(info, new BrushConverter(), null, null, null)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var border = new Border()
			{
				Margin = new Thickness(2, 0, 2, 0),
				BorderBrush = Brushes.Black,
				BorderThickness = new Thickness(1),
				CornerRadius = new CornerRadius(2),
			};

			ApplyBinding(border, Border.BackgroundProperty, GridColumnBindingTarget.Viewer);

			return border;
		}

		protected override bool ConfirmBeginEdit(RoutedEventArgs e)
		{
			return false;
		}

		protected override FrameworkElement CreateEditor(object rowData)
		{
			return null;
		}
	}

	class BrushConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			if (value is Color)
				return new SolidColorBrush((Color)value);

			return value;
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			throw new System.NotImplementedException();
		}
	}

	class SiteColumn : TextColumn
	{
		public SiteColumn(ItemPropertyInfo info)
			: base(info)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var viewer = base.CreateViewer(rowData) as TextBlock;
			if (viewer == null)
				return null;

			viewer.Foreground = Brushes.White;
			viewer.FontSize = 10;

			var border = new Border()
			{
				Margin = new Thickness(2, 0, 2, 0),
				CornerRadius = new CornerRadius(4),
				Padding = new Thickness(2, 1, 2, 1),
				Child = viewer,
				HorizontalAlignment = HorizontalAlignment.Center,
				VerticalAlignment = VerticalAlignment.Center,
			};

			var value = this.GetPropertyValue(rowData) as string;
			if (value != null)
			{
				if (value == "C017")
					border.Background = new SolidColorBrush(Color.FromArgb(255, 130, 163, 198));
				else
					border.Background = new SolidColorBrush(Color.FromArgb(255, 158, 48, 61));
			}

			return border;
		}
	}

	class PatientColumn : TextColumn
	{
		public PatientColumn(ItemPropertyInfo info)
			: base(info)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var viewer = base.CreateViewer(rowData) as TextBlock;
			if (viewer != null)
				viewer.FontWeight = FontWeights.Bold;

			return viewer;
		}
	}

	class DrugColumn : TextColumn
	{
		public DrugColumn(ItemPropertyInfo info)
			: base(info)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var viewer = base.CreateViewer(rowData) as TextBlock;
			if (viewer == null)
				return null;

			StackPanel panel = new StackPanel()
			{
				Orientation = Orientation.Horizontal,
			};
			panel.Children.Add(viewer);

			var data = rowData as Data;
			if (data.HasAttachment)
			{
				panel.Children.Add(new Image()
				{
					Width = 16,
					Height = 16,
					Source = new BitmapImage(new Uri("/WpfApplication7;component/Resources/paperclip.png", UriKind.Relative)),
					Margin = new Thickness(2, 0, 2, 0),
				});
			}
			if (data.HasPrescription)
			{
				panel.Children.Add(new Image()
				{
					Width = 16,
					Height = 16,
					Source = new BitmapImage(new Uri("/WpfApplication7;component/Resources/tube.png", UriKind.Relative)),
					Margin = new Thickness(2, 0, 2, 0),
				});
			}

			return panel;
		}
	}

	class QtyColumn : TextColumn
	{
		public QtyColumn(ItemPropertyInfo info)
			: base(info)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var viewer = base.CreateViewer(rowData) as TextBlock;
			if (viewer == null)
				return null;

			viewer.HorizontalAlignment = HorizontalAlignment.Center;
			viewer.VerticalAlignment = VerticalAlignment.Center;
			viewer.Foreground = new SolidColorBrush(Color.FromArgb(255, 199, 73, 196));
			viewer.FontWeight = FontWeights.Bold;
			viewer.FontSize = 11;

			return viewer;
		}
	}

	class TypeColumn : ChoiceColumn
	{
		public TypeColumn(ItemPropertyInfo info)
			: base(info)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var viewer = base.CreateViewer(rowData) as TextBlock;
			if (viewer == null)
				return null;

			var type = (DataType)this.GetPropertyValue(rowData);
			switch (type)
			{
				case DataType.ChangeRx:
					viewer.Text = "Change - Rx.";
					viewer.Foreground = new SolidColorBrush(Color.FromArgb(255, 131, 37, 37));
					break;
				case DataType.ChangeSch:
					viewer.Text = "Change - Sch.";
					viewer.Foreground = new SolidColorBrush(Color.FromArgb(255, 131, 37, 37));
					break;
				case DataType.Extend:
					viewer.Foreground = new SolidColorBrush(Color.FromArgb(255, 104, 51, 159));
					break;
				case DataType.New:
					viewer.Text = "NEW!";
					viewer.Background = Brushes.Yellow;
					break;
				case DataType.Refill:
					viewer.Foreground = new SolidColorBrush(Color.FromArgb(255, 65, 163, 112));
					break;
			}

			viewer.HorizontalAlignment = HorizontalAlignment.Center;
			viewer.VerticalAlignment = VerticalAlignment.Center;
			viewer.FontWeight = FontWeights.Bold;
			viewer.FontSize = 11;

			return viewer;
		}
	}

	class StatusColumn : ChoiceColumn
	{
		public StatusColumn(ItemPropertyInfo info)
			: base(info)
		{
		}

		protected override FrameworkElement CreateViewer(object rowData)
		{
			var viewer = base.CreateViewer(rowData) as TextBlock;
			if (viewer == null)
				return null;

			viewer.Foreground = Brushes.White;
			viewer.FontWeight = FontWeights.Bold;
			viewer.FontSize = 11;
			viewer.HorizontalAlignment = HorizontalAlignment.Center;
			viewer.VerticalAlignment = VerticalAlignment.Center;

			var border = new Border()
			{
				CornerRadius = new CornerRadius(10),
				Child = viewer,
			};

			var status = (DataStatus)this.GetPropertyValue(rowData);
			switch (status)
			{
				case DataStatus.Cancelled:
					border.Background = new SolidColorBrush(Color.FromArgb(255, 254, 72, 58));
					break;
				case DataStatus.Rescheduled:
					border.Background = new SolidColorBrush(Color.FromArgb(255, 227, 150, 18));
					break;
				case DataStatus.Confirmed:
					border.Background = new SolidColorBrush(Color.FromArgb(255, 0, 183, 77));
					break;
			}

			return border;
		}
	}
}
